""" 
A python class for *non oriented* weighted graphs
L. Gonnord nov 2016. Last modified : May 2020.
"""

from LibGraphes import Graph

debug = False


def minimum_distance(dist, Q):
    ''' Auxiliary function for Dijkstra
    returns the minimal distance from the distinguished vertex (implicit)
    to a node in the Q set
    '''
    if debug : 
        print(Q, dist)
    return min([(dist[node], node)
                for node in Q if node is not None])[1]


class WeightedGraph(Graph):

    def __init__(self, graph_dict=None, weight_dict=None):
        ''' Constructor of weighted graphs,
        derive from its mother'''
        super(WeightedGraph, self).__init__(graph_dict)
        if weight_dict is None:
            weight_dict = {}
        self.__weight_dict = weight_dict

    def my_weight(self, v1, v2):
        ''' Gives the weight of a given edge'''
        if (v1,v2) in self.__weight_dict:
            return self.__weight_dict[(v1,v2)]
        else:
            return self.__weight_dict[(v2,v1)]
            
    #true iff all edges have a weight
    def verify_weights(self):
        #        print self.__weight_dict
        for edge in self.edges():
            (v1, v2) = list(edge)[0], list(edge)[1]
            if (v1, v2) not in self.__weight_dict and (v2, v1) not in self.__weight_dict:
                return False
        return True

    def do_dijkstra(self, source):
        ''' Minimal distance from the source, for all nodes'''
        vertices = self.vertices()
        edges = self.edges()
        dist = dict()
        previous = dict()

        # Gives a special value "inf" which is not of type int, shame!
        for vertex in vertices:
            dist[vertex] = float("inf")
            previous[vertex] = None

        dist[source] = 0
        Q = set(vertices)

        # TODO : implement Dijkstra !
        return (previous, dist)
